@@ -1,4 +1,3 @@
-{-# LANGUAGE CPP                  #-}
 {-# LANGUAGE MagicHash            #-}
 {-# LANGUAGE PolyKinds            #-}
 {-# LANGUAGE UndecidableInstances #-}
@@ -30,7 +29,7 @@ module Data.Vector.Fixed.Cont (
   , shuffleFun
   , withFun
   , dimapFun
-    -- * Vector type class
+    -- * Vector type Boxedclass
   , Dim
   , Vector(..)
   , length
@@ -126,9 +125,11 @@ import Data.Data             (Data)
 import Data.Kind             (Type)
 import Data.Functor.Identity (Identity(..))
 import Data.Typeable         (Proxy(..))
-import qualified Data.Foldable    as F
-import qualified Data.Traversable as T
-import Unsafe.Coerce       (unsafeCoerce)
+import Data.Foldable         qualified as F
+import Data.Traversable      qualified as T
+import Data.List.NonEmpty    qualified as NE
+import Data.Foldable1        qualified as F1
+import Unsafe.Coerce         (unsafeCoerce)
 import GHC.TypeLits
 import GHC.Exts       (Proxy#, proxy#)
 import Prelude        ( Bool(..), Int, Maybe(..), Either(..)
@@ -136,7 +137,7 @@ import Prelude        ( Bool(..), Int, Maybe(..), Either(..)
                       , Semigroup(..), Monoid(..)
                       , (.), ($), (&&), (||), (<$>), id, error, otherwise, fst
                       )
-
+import Data.Vector.Fixed.Compat
 
 ----------------------------------------------------------------
 -- Naturals
@@ -434,7 +435,7 @@ dimapFun fA fR fun
 ----------------------------------------------------------------
 
 -- | Size of vector expressed as Peano natural.
-type family Dim (v :: Type -> Type) :: PeanoNum
+type family Dim (v :: k) :: PeanoNum
 
 -- | Type class for vectors with fixed length. Instance should provide
 --   two functions: one to create vector from @N@ elements and another
@@ -477,7 +478,8 @@ length _ = peanoToInt (proxy# @(Dim v))
 --   Church encoded N-element vector.
 newtype ContVec n a = ContVec (forall r. Fun n a r -> r)
 
-type instance Dim (ContVec n) = n
+type instance Dim (ContVec n)   = n
+type instance Dim (ContVec n a) = n
 
 -- | Cons values to the @ContVec@.
 consPeano :: a -> ContVec n a -> ContVec ('S n) a
@@ -532,6 +534,7 @@ instance (ArityPeano n) => F.Foldable (ContVec n) where
   toList     = toList
   sum        = sum
   product    = foldl' (*) 0
+  length     = length
   {-# INLINE foldMap' #-}
   {-# INLINE foldr    #-}
   {-# INLINE foldl    #-}
@@ -539,11 +542,27 @@ instance (ArityPeano n) => F.Foldable (ContVec n) where
   {-# INLINE toList   #-}
   {-# INLINE sum      #-}
   {-# INLINE product  #-}
--- GHC<9.2 fails to compile this
-#if MIN_VERSION_base(4,16,0)
-  length = length
   {-# INLINE length #-}
-#endif
+
+
+instance (ArityPeano n, n ~ S k) => F1.Foldable1 (ContVec n) where
+  fold1        = foldl1 (<>)
+  foldMap1   f = foldl1  (<>) . map f
+  foldMap1'  f = foldl1' (<>) . map f
+  toNonEmpty v = dictionaryPred (proxy# @n)
+               $ head v NE.:| toList (tail v)
+  maximum = maximum
+  minimum = minimum
+  head    = head
+  last    = F1.last . F1.toNonEmpty
+  {-# INLINE fold1      #-}
+  {-# INLINE foldMap1   #-}
+  {-# INLINE foldMap1'  #-}
+  {-# INLINE toNonEmpty #-}
+  {-# INLINE maximum    #-}
+  {-# INLINE minimum    #-}
+  {-# INLINE head       #-}
+  {-# INLINE last       #-}
 
 instance (ArityPeano n) => T.Traversable (ContVec n) where
   sequence  = sequence
@@ -1013,7 +1032,6 @@ head
   $ runContVec
   $ uncurryFirst pure
 
-
 -- | /O(n)/ Get value at specified index.
 index :: ArityPeano n => Int -> ContVec n a -> a
 {-# INLINE index #-}
@@ -1284,7 +1302,8 @@ gfoldlF f c0 = accum
 -- Instances
 ----------------------------------------------------------------
 
-type instance Dim Complex = N2
+type instance Dim Complex     = N2
+type instance Dim (Complex a) = N2
 
 instance Vector Complex a where
   construct = Fun (:+)
@@ -1293,7 +1312,8 @@ instance Vector Complex a where
   {-# INLINE inspect #-}
 
 
-type instance Dim Identity = N1
+type instance Dim Identity     = N1
+type instance Dim (Identity a) = N1
 
 instance Vector Identity a where
   construct = Fun Identity
@@ -1302,7 +1322,8 @@ instance Vector Identity a where
   {-# INLINE inspect #-}
 
 
-type instance Dim ((,) a) = N2
+type instance Dim ((,) a)   = N2
+type instance Dim ((,) a b) = N2
 
 -- | Note this instance (and other instances for tuples) is
 --   essentially monomorphic in element type. Vector type /v/ of 2
@@ -1315,7 +1336,8 @@ instance (b~a) => Vector ((,) b) a where
   {-# INLINE inspect #-}
 
 
-type instance Dim ((,,) a b) = N3
+type instance Dim ((,,) a b)   = N3
+type instance Dim ((,,) a b c) = N3
 
 instance (b~a, c~a) => Vector ((,,) b c) a where
   construct = Fun (,,)
@@ -1324,7 +1346,8 @@ instance (b~a, c~a) => Vector ((,,) b c) a where
   {-# INLINE inspect #-}
 
 
-type instance Dim ((,,,) a b c) = N4
+type instance Dim ((,,,) a b c)   = N4
+type instance Dim ((,,,) a b c d) = N4
 
 instance (b~a, c~a, d~a) => Vector ((,,,) b c d) a where
   construct = Fun (,,,)
@@ -1333,7 +1356,8 @@ instance (b~a, c~a, d~a) => Vector ((,,,) b c d) a where
   {-# INLINE inspect #-}
 
 
-type instance Dim ((,,,,) a b c d) = N5
+type instance Dim ((,,,,) a b c d)   = N5
+type instance Dim ((,,,,) a b c d e) = N5
 
 instance (b~a, c~a, d~a, e~a) => Vector ((,,,,) b c d e) a where
   construct = Fun (,,,,)
@@ -1342,7 +1366,8 @@ instance (b~a, c~a, d~a, e~a) => Vector ((,,,,) b c d e) a where
   {-# INLINE inspect #-}
 
 
-type instance Dim ((,,,,,) a b c d e) = N6
+type instance Dim ((,,,,,) a b c d e)   = N6
+type instance Dim ((,,,,,) a b c d e f) = N6
 
 instance (b~a, c~a, d~a, e~a, f~a) => Vector ((,,,,,) b c d e f) a where
   construct = Fun (,,,,,)
@@ -1351,7 +1376,8 @@ instance (b~a, c~a, d~a, e~a, f~a) => Vector ((,,,,,) b c d e f) a where
   {-# INLINE inspect #-}
 
 
-type instance Dim ((,,,,,,) a b c d e f) = N7
+type instance Dim ((,,,,,,) a b c d e f)   = N7
+type instance Dim ((,,,,,,) a b c d e f g) = N7
 
 instance (b~a, c~a, d~a, e~a, f~a, g~a) => Vector ((,,,,,,) b c d e f g) a where
   construct = Fun (,,,,,,)
@@ -1359,7 +1385,8 @@ instance (b~a, c~a, d~a, e~a, f~a, g~a) => Vector ((,,,,,,) b c d e f g) a where
   {-# INLINE construct #-}
   {-# INLINE inspect #-}
 
-type instance Dim Proxy = Z
+type instance Dim Proxy     = Z
+type instance Dim (Proxy a) = Z
 
 instance Vector Proxy a where
   construct = Fun Proxy
