@@ -38,7 +38,8 @@ import Data.Monoid           (Monoid(..),Dual(..),Sum(..),Product(..),All(..),An
 import Data.Semigroup        (Semigroup(..))
 import Data.Ord              (Down(..))
 import Data.Word             (Word,Word8,Word16,Word32,Word64)
-import Foreign.Storable      (Storable(..))
+import Data.Primitive.Types  (Prim)
+import Foreign.Storable      (Storable)
 import GHC.TypeLits
 import GHC.Exts              (Proxy#, proxy#)
 import Prelude               ( Show(..),Eq(..),Ord(..),Num(..),Applicative(..)
@@ -46,7 +47,9 @@ import Prelude               ( Show(..),Eq(..),Ord(..),Num(..),Applicative(..)
 
 import Data.Vector.Fixed           (Dim,Vector(..),ViaFixed(..))
 import Data.Vector.Fixed           qualified as F
+import Data.Vector.Fixed.Compat
 import Data.Vector.Fixed.Cont      qualified as C
+import Data.Vector.Fixed.Mono      qualified as FM
 import Data.Vector.Fixed.Cont      (Peano,Arity,ArityPeano,Fun(..),curryFirst)
 import Data.Vector.Fixed.Primitive qualified as P
 
@@ -85,7 +88,8 @@ class ( Dim    (VecRepr n a) ~ Peano n
   -- | Convert element from its representation
   fromEltRepr :: Proxy# n -> EltRepr a -> a
 
-type instance Dim (Vec n) = Peano n
+type instance Dim (Vec n)   = Peano n
+type instance Dim (Vec n a) = Peano n
 
 instance (Arity n, Unbox n a) => Vector (Vec n) a where
   inspect (Vec v) f
@@ -97,6 +101,14 @@ instance (Arity n, Unbox n a) => Vector (Vec n) a where
   {-# INLINE inspect   #-}
   {-# INLINE construct #-}
 
+instance (Arity n, Unbox n a) => FM.Prod a (Vec n a) where
+  construct  = construct
+  inspect    = inspect
+  {-# INLINE construct  #-}
+  {-# INLINE inspect    #-}
+
+instance (Arity n, Unbox n a) => FM.Vector a (Vec n a)
+
 
 
 ----------------------------------------------------------------
@@ -110,6 +122,8 @@ deriving via ViaFixed (Vec n) a instance (Unbox n a, NFData    a) => NFData    (
 deriving via ViaFixed (Vec n) a instance (Unbox n a, Semigroup a) => Semigroup (Vec n a)
 deriving via ViaFixed (Vec n) a instance (Unbox n a, Monoid    a) => Monoid    (Vec n a)
 deriving via ViaFixed (Vec n) a instance (Unbox n a, Storable  a) => Storable  (Vec n a)
+-- | @since 2.0.1.0
+deriving via ViaFixed (Vec n) a instance (Unbox n a, Prim      a) => Prim      (Vec n a)
 
 instance (Typeable n, Unbox n a, Data a) => Data (Vec n a) where
   gfoldl       = C.gfoldl
@@ -138,7 +152,8 @@ instance F.Arity n => Unbox n () where
 
 data VecUnit (n :: Nat) a = VecUnit
 
-type instance Dim (VecUnit n) = Peano n
+type instance Dim (VecUnit n)   = Peano n
+type instance Dim (VecUnit n a) = Peano n
 
 instance F.Arity n => Vector (VecUnit n) () where
   inspect _ fun
@@ -159,7 +174,8 @@ instance F.Arity n => Vector (VecUnit n) () where
 --   GHC quite a bit.
 data BitVec (n :: Nat) a = BitVec Word64
 
-type instance Dim (BitVec n) = Peano n
+type instance Dim (BitVec n)   = Peano n
+type instance Dim (BitVec n a) = Peano n
 
 instance (n <= 64, Arity n, a ~ Bool) => Vector (BitVec n) a where
   inspect (BitVec w) = inspect (C.generate (testBit w))
@@ -235,7 +251,8 @@ deriving newtype instance (n <= 64, Arity n) => Unbox n Any
 -- | Representation for vector of 2-tuple as two vectors.
 data T2 n a b x = T2 !(Vec n a) !(Vec n b)
 
-type instance Dim (T2 n a b) = Peano n
+type instance Dim (T2 n a b)   = Peano n
+type instance Dim (T2 n a b x) = Peano n
 
 instance (Arity n, Unbox n a, Unbox n b) => Vector (T2 n a b) (a,b) where
   inspect (T2 vA vB)
@@ -265,7 +282,8 @@ data T_pair a b x y n = T_pair (Fun n a x) (Fun n b y)
 -- | Representation for vector of 2-tuple as two vectors.
 data T3 n a b c x = T3 !(Vec n a) !(Vec n b) !(Vec n c)
 
-type instance Dim (T3 n a b c) = Peano n
+type instance Dim (T3 n a b c)   = Peano n
+type instance Dim (T3 n a b c x) = Peano n
 
 instance (Arity n, Unbox n a, Unbox n b, Unbox n c) => Vector (T3 n a b c) (a,b,c) where
   inspect (T3 vA vB vC)
